#! /usr/bin/env python
"""
Usage:

   bam-coverage.py reference.fa query.x.reference.bam minmatch mapq

bam-coverage calculates the fraction of bases in 'reference.fa' that are
covered by sequences in a BAM alignment file that are longer than 'minmatch'.

Original script: https://github.com/ngs-docs/ngs-scripts/blob/master/blast/calc-blast-cover.py
"""

from __future__ import print_function, division
import sys

DEPENDENCIES = {
    "pysam": False,
    "screed": False
}

try:
    import pysam
    DEPENDENCIES['pysam'] = True
except ImportError:
    pass

try:
    import screed
    DEPENDENCIES['screed'] = True
except ImportError:
    pass


def check_dependencies():
    if all(DEPENDENCIES[d] for d in DEPENDENCIES):
        return True

    print("Missing dependencies, install before proceeding:", file=sys.stderr)
    for dep, installed in DEPENDENCIES.items():
        if not installed:
            print(dep, file=sys.stderr)

    sys.exit(1)


def bam_coverage(reference, alignments, min_match, min_mapq=30, min_len=0):
    check_dependencies()

    # create empty lists representing the total number of bases in the
    # reference
    print("creating empty lists", file=sys.stderr)
    covs = {}
    for record in screed.open(reference):
        covs[record.name] = [0] * len(record.sequence)

    # run through the BAM records in the query, and calculate how much of
    # the reference is covered by the query.
    print('building coverage', file=sys.stderr)
    with pysam.Samfile(alignments, "rb") as samfile:
        for record in samfile:

            if len(record.query) < min_match:
                continue

            if record.mapq < min_mapq:
                continue

            cov = covs.get(samfile.getrname(record.tid))
            if not cov:
                continue

            if min_len and len(record.aligned_pairs) < min_len * len(record.seq):
                continue

            for pos_read, pos_ref in record.aligned_pairs:
                if pos_ref:
                    cov[pos_ref] = 1

    # print out summary statistics for each of the reference.
    coved = {}
    sizes = {}
    total = 0
    covered = 0
    print('Summing stats', file=sys.stderr)
    for name in covs:
        coved[name] = sum(covs[name])
        sizes[name] = float(len(covs[name]))
        covered += coved[name]
        total += sizes[name]
    fraction = covered / float(total or 1)

    print('total bases in reference:', total)
    print('total ref bases covered :', covered)
    print('fraction                :', fraction)
    print('reference               :', reference)
    print('BAM alignment file      :', alignments)

    return {
        'total': total,
        'covered': covered,
        'fraction': fraction,
        'coverage per contig': coved,
        'contig size': sizes
    }


if __name__ == "__main__":
    reference = sys.argv[1]
    alignments = sys.argv[2]
    min_match = int(sys.argv[3])
    query = sys.argv[4]
    min_mapq = int(sys.argv[5])

    bam_coverage(reference, alignments, min_match, query, min_mapq=min_mapq)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import shutil
import unittest

from nose.tools import *  # PEP8 asserts
from scripttest import TestFileEnvironment


class TestBamConverage(unittest.TestCase):

    '''Unit tests for bam_coverage.'''

    def setUp(self):
        self.env = TestFileEnvironment()
        shutil.copytree(os.path.join(os.path.dirname(__file__), 'data'),
                        os.path.join(self.env.base_path, 'data'))
        self.total = 237
        self.coved = 154
        self.fraction = 0.6497
        self.min_match_coved = 115
        self.min_match_fraction = 0.485

    def tearDown(self):
        self.env.clear()

    def test_arguments(self):
        ret = self.env.run("bioinfo", "-h")
        self.assertEquals(ret.returncode, 0)
        self.assertIn('bam_coverage', ret.stdout)

    def test_complete_run_cli(self):
        ret = self.env.run("bioinfo", "bam_coverage",
                           "data/bam_ref.fasta",
                           "data/bam_align.sorted.bam",
                           "10",
                           "--mapq=30", expect_stderr=True)
        self.assertEquals(ret.returncode, 0)

        self.assertIn('total bases in reference', ret.stdout)
        self.assertIn('total ref bases covered', ret.stdout)
        self.assertIn('fraction', ret.stdout)

        self.assertNotIn('total bases in reference', ret.stderr)
        self.assertNotIn('total ref bases covered', ret.stderr)
        self.assertNotIn('fraction', ret.stderr)

        for line in ret.stdout.split('\n'):
            if 'total bases in reference' in line:
                self.assertIn(str(self.total), line)
            if 'total ref bases covered' in line:
                self.assertIn(str(self.coved), ret.stdout)
            if 'fraction' in line:
                self.assertIn(str(self.fraction), ret.stdout)

    def test_complete_run_module(self):
        from bioinfo import bam_coverage

        result = bam_coverage(
            os.path.join('tests', 'data', 'bam_ref.fasta'),
            os.path.join('tests', 'data', 'bam_align.sorted.bam'),
            10,
            30
        )
        self.assertEqual(result['total'], self.total)
        self.assertEqual(result['covered'], self.coved)
        self.assertAlmostEqual(result['fraction'], self.fraction, places=3)

    def test_min_match(self):
        from bioinfo import bam_coverage

        result = bam_coverage(
            os.path.join('tests', 'data', 'bam_ref.fasta'),
            os.path.join('tests', 'data', 'bam_align.sorted.bam'),
            45,
            30
        )
        self.assertEqual(result['total'], self.total)
        self.assertEqual(result['covered'], self.min_match_coved)
        self.assertAlmostEqual(result['fraction'],
                               self.min_match_fraction,
                               places=2)

    def test_check_dependencies(self):
        import bioinfo
        from bioinfo import bam_coverage

        bioinfo.bam_coverage_mod.DEPENDENCIES['pysam'] = False

        with self.assertRaises(SystemExit):
            bam_coverage(
                os.path.join('tests', 'data', 'bam_ref.fasta'),
                os.path.join('tests', 'data', 'bam_align.sorted.bam'),
                10,
                30
            )

        bioinfo.bam_coverage_mod.DEPENDENCIES['pysam'] = True


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''bioinfo

Collection of scripts for bioinformatics problems.

Usage:
  bioinfo bam_coverage <reference> <alignments> <minmatch> [--mapq=<n>] [--minlen=<n>]
  bioinfo -h | --help
  bioinfo --version

Options:
  --mapq=<n>    Minimum MAPQ quality [default: 30].
  --minlen=<n>  Minimum alignment length, as a percentage of read length. Max 1.0 [default: 0].
  -h --help     Show this screen.
  --version     Show version.
'''

from __future__ import unicode_literals, print_function, division
from docopt import docopt

from bioinfo import bam_coverage as bam_coverage_mod

from bioinfo.bam_coverage import bam_coverage


__version__ = "0.1.8"
__author__ = "Luiz Irber"
__license__ = "BSD License"


AVAILABLE_COMMANDS = {
    'bam_coverage': bam_coverage,
}


def main():
    '''Main entry point for the bioinfo CLI.'''
    args = docopt(__doc__, version=__version__)

    if 'bam_coverage' in args:
        bam_coverage(args['<reference>'],
                     args['<alignments>'],
                     int(args['<minmatch>']),
                     min_mapq=int(args['--mapq']),
                     min_len=float(args['--minlen']))

if __name__ == '__main__':
    main()



import sys
import os
import subprocess

from setuptools import setup

PUBLISH_CMD = "python setup.py register sdist upload"
TEST_PUBLISH_CMD = 'python setup.py register -r test sdist upload -r test'
TEST_CMD = 'nosetests'

if 'publish' in sys.argv:
    status = subprocess.call(PUBLISH_CMD, shell=True)
    sys.exit(status)

if 'publish_test' in sys.argv:
    status = subprocess.call(TEST_PUBLISH_CMD, shell=True)
    sys.exit()

if 'run_tests' in sys.argv:
    try:
        __import__('nose')
    except ImportError:
        print('nose required. Run `pip install nose`.')
        sys.exit(1)
    status = subprocess.call(TEST_CMD, shell=True)
    sys.exit(status)

def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content

setup(
    name='bioinfo',
    version="0.1.8",
    description='Bioinformatics scripts',
    long_description=read("README.rst"),
    author='Luiz Irber',
    author_email='luiz@luizirber.org',
    url='https://github.com/luizirber/bioinfo',
    install_requires=['docopt'],
    license=read("LICENSE"),
    zip_safe=False,
    keywords='bioinfo',
    packages=['bioinfo'],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
    entry_points={
        'console_scripts': [
            "bioinfo = bioinfo:main"
        ]
    },
    tests_require=['nose'],
)
